@@ -666,12 +666,13 @@ set_markup (GkbdKeyboardDrawingRenderContext * context, gchar * txt)
 	}
 }
 
-static void
+static PangoDirection
 set_key_label_in_layout (GkbdKeyboardDrawingRenderContext * context,
 			 guint keyval)
 {
 	gchar buf[5];
 	gunichar uc;
+	PangoDirection dir = PANGO_DIRECTION_LTR;
 
 	switch (keyval) {
 	case GDK_KEY_Scroll_Lock:
@@ -825,6 +826,7 @@ set_key_label_in_layout (GkbdKeyboardDrawingRenderContext * context,
 	default:
 		uc = gdk_keyval_to_unicode (keyval);
 		if (uc != 0 && g_unichar_isgraph (uc)) {
+			dir = pango_unichar_direction (uc);
 			buf[g_unichar_to_utf8 (uc, buf)] = '\0';
 			set_markup (context, buf);
 		} else {
@@ -846,6 +848,7 @@ set_key_label_in_layout (GkbdKeyboardDrawingRenderContext * context,
 				set_markup (context, "");
 		}
 	}
+	return dir;
 }
 
 
@@ -927,6 +930,8 @@ draw_key_label_helper (GkbdKeyboardDrawingRenderContext * context,
 		       gboolean is_pressed)
 {
 	gint label_x, label_y, label_max_width, ycell;
+	PangoAlignment align;
+	PangoDirection dir;
 
 	if (keysym == 0)
 		return;
@@ -935,46 +940,36 @@ draw_key_label_helper (GkbdKeyboardDrawingRenderContext * context,
 		(unsigned) keysym, (char) keysym, (int) glp);
 #endif
 
+	ycell = ((glp == GKBD_KEYBOARD_DRAWING_POS_BOTTOMLEFT) ||
+		 (glp == GKBD_KEYBOARD_DRAWING_POS_BOTTOMRIGHT));
+	rotate_coordinate (x, y, x + padding,
+			   y + padding + (height -
+					  2 * padding) *
+			   ycell * 4 / 7, angle, &label_x,
+			   &label_y);
+	label_max_width = PANGO_SCALE * (width - 2 * padding);
+	dir = set_key_label_in_layout (context, keysym);
+
 	switch (glp) {
 	case GKBD_KEYBOARD_DRAWING_POS_TOPLEFT:
 	case GKBD_KEYBOARD_DRAWING_POS_BOTTOMLEFT:
-		{
-			ycell =
-			    glp == GKBD_KEYBOARD_DRAWING_POS_BOTTOMLEFT;
-
-			rotate_coordinate (x, y, x + padding,
-					   y + padding + (height -
-							  2 * padding) *
-					   ycell * 4 / 7, angle, &label_x,
-					   &label_y);
-			label_max_width =
-			    PANGO_SCALE * (width - 2 * padding);
-			break;
-		}
+		if (dir == PANGO_DIRECTION_RTL)
+			align = PANGO_ALIGN_RIGHT;
+		else
+			align = PANGO_ALIGN_LEFT;
+		break;
 	case GKBD_KEYBOARD_DRAWING_POS_TOPRIGHT:
 	case GKBD_KEYBOARD_DRAWING_POS_BOTTOMRIGHT:
-		{
-			ycell =
-			    glp == GKBD_KEYBOARD_DRAWING_POS_BOTTOMRIGHT;
-
-			rotate_coordinate (x, y,
-					   x + padding + (width -
-							  2 * padding) *
-					   4 / 7,
-					   y + padding + (height -
-							  2 * padding) *
-					   ycell * 4 / 7, angle, &label_x,
-					   &label_y);
-			label_max_width =
-			    PANGO_SCALE * ((width - 2 * padding) -
-					   (width - 2 * padding) * 4 / 7);
-			break;
-		}
+		if (dir == PANGO_DIRECTION_RTL)
+			align = PANGO_ALIGN_LEFT;
+		else
+			align = PANGO_ALIGN_RIGHT;
+		break;
 	default:
 		return;
 	}
-	set_key_label_in_layout (context, keysym);
 	pango_layout_set_width (context->layout, label_max_width);
+	pango_layout_set_alignment (context->layout, align);
 	label_y -= (pango_layout_get_line_count (context->layout) - 1) *
 	    (pango_font_description_get_size (context->font_desc) /
 	     PANGO_SCALE);
@@ -2017,15 +2012,6 @@ gkbd_keyboard_drawing_init (GkbdKeyboardDrawing * drawing)
 		g_critical
 		    ("XkbQueryExtension failed! Stuff probably won't work.");
 
-	/* XXX: this stuff probably doesn't matter.. also, gdk_screen_get_default can fail */
-	if (gtk_widget_has_screen (GTK_WIDGET (drawing)))
-		drawing->screen_num =
-		    gdk_screen_get_number (gtk_widget_get_screen
-					   (GTK_WIDGET (drawing)));
-	else
-		drawing->screen_num =
-		    gdk_screen_get_number (gdk_screen_get_default ());
-
 	alloc_render_context (drawing);
 
 	drawing->keyboard_items = NULL;
@@ -2124,13 +2110,9 @@ get_preferred_width (GtkWidget * widget,
 		     gint * minimum_width, gint * natural_width)
 {
 	GdkRectangle rect;
-	gint w, monitor;
-	GdkScreen *scr = NULL;
+	gint w;
 
-	scr = gdk_screen_get_default ();
-	monitor = gdk_screen_get_primary_monitor (scr);
-
-	gdk_screen_get_monitor_geometry (scr, monitor, &rect);
+	gdk_monitor_get_geometry (gdk_display_get_primary_monitor (gdk_display_get_default ()), &rect);
 	w = rect.width;
 	*minimum_width = *natural_width = w - (w >> 2);
 }
@@ -2142,6 +2124,10 @@ get_preferred_height_for_width (GtkWidget * widget,
 				gint * natural_height)
 {
 	GkbdKeyboardDrawing *drawing = GKBD_KEYBOARD_DRAWING (widget);
+
+	if (!drawing->xkb)
+		return;
+
 	*minimum_height = *natural_height =
 	    width * drawing->xkb->geom->height_mm /
 	    drawing->xkb->geom->width_mm;
@@ -2315,7 +2301,7 @@ gkbd_keyboard_drawing_set_keyboard (GkbdKeyboardDrawing * drawing,
 	return TRUE;
 }
 
-G_CONST_RETURN gchar *
+const gchar *
 gkbd_keyboard_drawing_get_keycodes (GkbdKeyboardDrawing * drawing)
 {
 	if (!drawing->xkb || drawing->xkb->names->keycodes <= 0)
@@ -2325,7 +2311,7 @@ gkbd_keyboard_drawing_get_keycodes (GkbdKeyboardDrawing * drawing)
 				     drawing->xkb->names->keycodes);
 }
 
-G_CONST_RETURN gchar *
+const gchar *
 gkbd_keyboard_drawing_get_geometry (GkbdKeyboardDrawing * drawing)
 {
 	if (!drawing->xkb || drawing->xkb->names->geometry <= 0)
@@ -2335,7 +2321,7 @@ gkbd_keyboard_drawing_get_geometry (GkbdKeyboardDrawing * drawing)
 				     drawing->xkb->names->geometry);
 }
 
-G_CONST_RETURN gchar *
+const gchar *
 gkbd_keyboard_drawing_get_symbols (GkbdKeyboardDrawing * drawing)
 {
 	if (!drawing->xkb || drawing->xkb->names->symbols <= 0)
@@ -2345,7 +2331,7 @@ gkbd_keyboard_drawing_get_symbols (GkbdKeyboardDrawing * drawing)
 				     drawing->xkb->names->symbols);
 }
 
-G_CONST_RETURN gchar *
+const gchar *
 gkbd_keyboard_drawing_get_types (GkbdKeyboardDrawing * drawing)
 {
 	if (!drawing->xkb || drawing->xkb->names->types <= 0)
@@ -2355,7 +2341,7 @@ gkbd_keyboard_drawing_get_types (GkbdKeyboardDrawing * drawing)
 				     drawing->xkb->names->types);
 }
 
-G_CONST_RETURN gchar *
+const gchar *
 gkbd_keyboard_drawing_get_compat (GkbdKeyboardDrawing * drawing)
 {
 	if (!drawing->xkb || drawing->xkb->names->compat <= 0)
@@ -2454,7 +2440,7 @@ gkbd_keyboard_drawing_draw_page (GtkPrintOperation * operation,
 	gtk_print_operation_set_unit (operation, GTK_UNIT_PIXEL);
 
 	header = g_strdup_printf
-	    (_("Keyboard layout \"%s\"\n"
+	    (_("Keyboard layout “%s”\n"
 	       "Copyright &#169; X.Org Foundation and "
 	       "XKeyboardConfig contributors\n"
 	       "For licensing see package metadata"), data->description);
